from __future__ import print_function
from __future__ import absolute_import
# Copyright 2013-2014 Amazon.com, Inc. or its affiliates. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License"). You may not use this file except in compliance with the
# License. A copy of the License is located at
#
# http://aws.amazon.com/apache2.0/
#
# or in the "LICENSE.txt" file accompanying this file. This file is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES
# OR CONDITIONS OF ANY KIND, express or implied. See the License for the specific language governing permissions and
# limitations under the License.

from future import standard_library
standard_library.install_aliases()
from builtins import str
from builtins import object
import configparser
import os
import sys
import inspect
import pkg_resources
import json
import urllib.request, urllib.error, urllib.parse
from . import config_sanity
import boto3
from botocore.exceptions import ClientError

def getStackTemplate(region, aws_access_key_id, aws_secret_access_key, stack):
    cfn = boto3.client('cloudformation', region_name=region,
                       aws_access_key_id=aws_access_key_id,
                       aws_secret_access_key=aws_secret_access_key)
    __stack_name = ('cfncluster-' + stack)

    try:
        __stack = cfn.describe_stacks(StackName=__stack_name).get('Stacks')[0]
    except ClientError as e:
        print(e.response.get('Error').get('Message'))
        sys.stdout.flush()
        sys.exit(1)
    __cli_template = [p.get('ParameterValue') for p in __stack.get('Parameters') if p.get('ParameterKey') == 'CLITemplate'][0]

    return __cli_template

class CfnClusterConfig(object):

    def __init__(self, args):
        self.args = args
        self.parameters = []
        self.version = pkg_resources.get_distribution("cfncluster").version
        self.__DEFAULT_CONFIG = False
        __args_func = self.args.func.__name__

        # Determine config file name based on args or default
        if args.config_file is not None:
            self.__config_file = args.config_file
        else:
            self.__config_file = os.path.expanduser(os.path.join('~', '.cfncluster', 'config'))
            self.__DEFAULT_CONFIG = True
        if os.path.isfile(self.__config_file):
            pass
        else:
            if self.__DEFAULT_CONFIG:
                print('Default config %s not found' % self.__config_file)
                print('You can copy a template from here: %s%sexamples%sconfig' %
                      (os.path.dirname(os.path.abspath(inspect.getfile(inspect.currentframe()))),
                       os.path.sep, os.path.sep))
                sys.exit(1)
            else:
                print('Config file %s not found' % self.__config_file)
                sys.exit(1)


        __config = configparser.ConfigParser()
        __config.read(self.__config_file)

        # Determine the EC2 region to used used or default to us-east-1
        # Order is 1) CLI arg 2) AWS_DEFAULT_REGION env 3) Config file 4) us-east-1
        if args.region:
            self.region = args.region
        else:
            if os.environ.get('AWS_DEFAULT_REGION'):
                self.region = os.environ.get('AWS_DEFAULT_REGION')
            else:
                try:
                    self.region = __config.get('aws', 'aws_region_name')
                except configparser.NoOptionError:
                    self.region = 'us-east-1'

        # Check if credentials have been provided in config
        try:
            self.aws_access_key_id = __config.get('aws', 'aws_access_key_id')
        except configparser.NoOptionError:
            self.aws_access_key_id=None
        try:
            self.aws_secret_access_key = __config.get('aws', 'aws_secret_access_key')
        except configparser.NoOptionError:
            self.aws_secret_access_key=None

        # Determine which cluster template will be used
        if __args_func == 'start':
            # Starting a cluster is unique in that we would want to prevent the
            # customer from inadvertently using a different template than what
            # the cluster was created with, so we do not support the -t
            # parameter. We always get the template to use from CloudFormation.
            self.__cluster_template = getStackTemplate(self.region,self.aws_access_key_id,
                                        self.aws_secret_access_key, self.args.cluster_name)
        else:
            try:
                if args.cluster_template is not None:
                    self.__cluster_template = args.cluster_template
                else:
                    if __args_func == 'update':
                        self.__cluster_template = getStackTemplate(self.region,self.aws_access_key_id,
                                                                   self.aws_secret_access_key, self.args.cluster_name)
                    else:
                        self.__cluster_template = __config.get('global', 'cluster_template')
            except AttributeError:
                self.__cluster_template = __config.get('global', 'cluster_template')
        self.__cluster_section = ('cluster %s' % self.__cluster_template)
        self.parameters.append(('CLITemplate',self.__cluster_template))

        # Check if package updates should be checked
        try:
            self.__update_check = __config.getboolean('global', 'update_check')
        except configparser.NoOptionError:
            self.__update_check = True

        if self.__update_check == True:
            try:
                __latest = json.loads(urllib.request.urlopen("http://pypi.python.org/pypi/cfncluster/json").read())['info']['version']
                if self.version < __latest:
                    print('warning: There is a newer version %s of cfncluster available.' % __latest)
            except Exception:
                pass

        # Check if config sanity should be run
        try:
            self.__sanity_check = __config.getboolean('global', 'sanity_check')
        except configparser.NoOptionError:
            self.__sanity_check = False
        # Only check config on calls that mutate it
        __args_func = self.args.func.__name__
        if (__args_func == 'create' or __args_func == 'update' or __args_func == 'configure') and self.__sanity_check is True:
            pass
        else:
            self.__sanity_check = False

        # Get the EC2 keypair name to be used, exit if not set
        try:
            self.key_name = __config.get(self.__cluster_section, 'key_name')
            if not self.key_name:
                print("ERROR: key_name set in [%s] section but not defined." % self.__cluster_section)
                sys.exit(1)
            if self.__sanity_check:
                config_sanity.check_resource(self.region,self.aws_access_key_id, self.aws_secret_access_key,
                                             'EC2KeyPair', self.key_name)
        except configparser.NoOptionError:
            print("ERROR: Missing key_name option in [%s] section." % self.__cluster_section)
            sys.exit(1)
        self.parameters.append(('KeyName', self.key_name))

        # Determine the CloudFormation URL to be used
        # Order is 1) CLI arg 2) Config file 3) default for version + region
        try:
            if args.template_url is not None:
                self.template_url = args.template_url
            else:
                try:
                    self.template_url = __config.get(self.__cluster_section,
                                                     'template_url')
                    if not self.template_url:
                        print("ERROR: template_url set in [%s] section but not defined." % self.__cluster_section)
                        sys.exit(1)
                    if self.__sanity_check:
                        config_sanity.check_resource(self.region, self.aws_access_key_id, self.aws_secret_access_key,
                                                     'URL', self.template_url)
                except configparser.NoOptionError:
                    if self.region == 'us-gov-west-1':
                        self.template_url = ('https://s3-%s.amazonaws.com/cfncluster-%s/templates/cfncluster-%s.cfn.json'
                                             % (self.region, self.region, self.version))
                    else:
                        self.template_url = ('https://s3.amazonaws.com/%s-cfncluster/templates/cfncluster-%s.cfn.json'
                                             % (self.region, self.version))
        except AttributeError:
            pass

        # Determine which vpc settings section will be used
        self.__vpc_settings = __config.get(self.__cluster_section, 'vpc_settings')
        self.__vpc_section = ('vpc %s' % self.__vpc_settings)

        # Dictionary list of all VPC options
        self.__vpc_options = dict(vpc_id=('VPCId','VPC'), master_subnet_id=('MasterSubnetId', 'VPCSubnet'),
                                  compute_subnet_cidr=('ComputeSubnetCidr',None),
                                  compute_subnet_id=('ComputeSubnetId', 'VPCSubnet'), use_public_ips=('UsePublicIps',
                                                                                                      None),
                                  ssh_from=('AccessFrom', None), access_from=('AccessFrom', None),
                                  additional_sg=('AdditionalSG','VPCSecurityGroup'),
                                  vpc_security_group_id=('VPCSecurityGroupId','VPCSecurityGroup')
                                  )

        # Loop over all VPC options and add define to parameters, raise Exception is defined but null
        for key in self.__vpc_options:
            try:
                __temp__ = __config.get(self.__vpc_section, key)
                if not __temp__:
                    print("ERROR: %s defined but not set in [%s] section"
                                                    % (key, self.__vpc_section))
                    sys.exit(1)
                if self.__sanity_check and self.__vpc_options.get(key)[1] is not None:
                    config_sanity.check_resource(self.region,self.aws_access_key_id, self.aws_secret_access_key,
                                                self.__vpc_options.get(key)[1],__temp__)
                self.parameters.append((self.__vpc_options.get(key)[0],__temp__))
            except configparser.NoOptionError:
                pass

        # Dictionary list of all cluster section options
        self.__cluster_options = dict(cluster_user=('ClusterUser', None), compute_instance_type=('ComputeInstanceType',None),
                                      master_instance_type=('MasterInstanceType', None), initial_queue_size=('InitialQueueSize',None),
                                      max_queue_size=('MaxQueueSize',None), maintain_initial_size=('MaintainInitialSize',None),
                                      scheduler=('Scheduler',None), cluster_type=('ClusterType',None), ephemeral_dir=('EphemeralDir',None),
                                      spot_price=('SpotPrice',None), custom_ami=('CustomAMI','EC2Ami'), pre_install=('PreInstallScript','URL'),
                                      post_install=('PostInstallScript','URL'), proxy_server=('ProxyServer',None),
                                      placement=('Placement',None), placement_group=('PlacementGroup','EC2PlacementGroup'),
                                      encrypted_ephemeral=('EncryptedEphemeral',None),pre_install_args=('PreInstallArgs',None),
                                      post_install_args=('PostInstallArgs',None), s3_read_resource=('S3ReadResource',None),
                                      s3_read_write_resource=('S3ReadWriteResource',None),cwl_region=('CWLRegion',None),
                                      cwl_log_group=('CWLLogGroup',None),shared_dir=('SharedDir',None),tenancy=('Tenancy',None),
                                      ephemeral_kms_key_id=('EphemeralKMSKeyId',None), cluster_ready=('ClusterReadyScript','URL'),
                                      master_root_volume_size=('MasterRootVolumeSize',None),compute_root_volume_size=('ComputeRootVolumeSize',None),
                                      base_os=('BaseOS',None),ec2_iam_role=('EC2IAMRoleName','EC2IAMRoleName'),extra_json=('ExtraJson',None),
                                      custom_chef_cookbook=('CustomChefCookbook',None),custom_chef_runlist=('CustomChefRunList',None),
                                      additional_cfn_template=('AdditionalCfnTemplate',None)
                                      )

        # Loop over all the cluster options and add define to parameters, raise Exception if defined but null
        for key in self.__cluster_options:
            try:
                __temp__ = __config.get(self.__cluster_section, key)
                if not __temp__:
                    print("ERROR: %s defined but not set in [%s] section"
                                                    % (key, self.__cluster_section))
                    sys.exit(1)
                if self.__sanity_check and self.__cluster_options.get(key)[1] is not None:
                    config_sanity.check_resource(self.region,self.aws_access_key_id, self.aws_secret_access_key,
                                                self.__cluster_options.get(key)[1],__temp__)
                self.parameters.append((self.__cluster_options.get(key)[0],__temp__))
            except configparser.NoOptionError:
                pass

        # Merge tags from config with tags from command line args
        # Command line args take precedent and overwite tags supplied in the config
        self.tags = {}
        try:
            tags = __config.get(self.__cluster_section, 'tags')
            self.tags = json.loads(tags);
        except configparser.NoOptionError:
            pass
        try:
            if args.tags is not None:
                for key in args.tags:
                    self.tags[key] = args.tags[key]
        except AttributeError:
            pass

        # Determine if EBS settings are defined and set section
        try:
            self.__ebs_settings = __config.get(self.__cluster_section, 'ebs_settings')
            if not self.__ebs_settings:
                print("ERROR: ebs_settings defined by not set in [%s] section"
                                                % self.__cluster_section)
                sys.exit(1)
            self.__ebs_section = ('ebs %s' % self.__ebs_settings)
        except configparser.NoOptionError:
            pass

        # Dictionary list of all EBS options
        self.__ebs_options = dict(ebs_snapshot_id=('EBSSnapshotId','EC2Snapshot'), volume_type=('VolumeType',None),
                                  volume_size=('VolumeSize',None), ebs_kms_key_id=('EBSKMSKeyId', None),
                                  volume_iops=('VolumeIOPS',None), encrypted=('EBSEncryption',None),
                                  ebs_volume_id=('EBSVolumeId','EC2Volume')
                                  )

        try:
            if self.__ebs_section:
                for key in self.__ebs_options:
                    try:
                        __temp__ = __config.get(self.__ebs_section, key)
                        if not __temp__:
                            print("ERROR: %s defined but not set in [%s] section"
                                                    % (key, self.__ebs_section))
                            sys.exit(1)
                        if self.__sanity_check and self.__ebs_options.get(key)[1] is not None:
                            config_sanity.check_resource(self.region,self.aws_access_key_id, self.aws_secret_access_key,
                                                self.__ebs_options.get(key)[1],__temp__)
                        self.parameters.append((self.__ebs_options.get(key)[0],__temp__))
                    except configparser.NoOptionError:
                        pass
        except AttributeError:
            pass

        # Determine if scaling settings are defined and set section
        try:
            self.__scaling_settings = __config.get(self.__cluster_section, 'scaling_settings')
            if not self.__scaling_settings:
                print("ERROR: scaling_settings defined by not set in [%s] section"
                                                % self.__cluster_section)
                sys.exit(1)
            self.__scaling_section = ('scaling %s' % self.__scaling_settings)
        except configparser.NoOptionError:
            pass

        # Dictionary list of all scaling options
        self.__scaling_options = dict(scaling_threshold=('ScalingThreshold',None), scaling_period=('ScalingPeriod',None),
                                      scaling_evaluation_periods=('ScalingEvaluationPeriods',None),
                                      scaling_adjustment=('ScalingAdjustment',None),scaling_adjustment2=('ScalingAdjustment2',None),
                                      scaling_cooldown=('ScalingCooldown',None),scaling_threshold2=('ScalingThreshold2',None))

        try:
            if self.__scaling_section:
                for key in self.__scaling_options:
                    try:
                        __temp__ = __config.get(self.__scaling_section, key)
                        if not __temp__:
                            print("ERROR: %s defined but not set in [%s] section"
                                                    % (key, self.__scaling_section))
                            sys.exit(1)
                        if self.__sanity_check and self.__scaling_options.get(key)[1] is not None:
                            config_sanity.check_resource(self.region,self.aws_access_key_id, self.aws_secret_access_key,
                                                self.__scaling_options.get(key)[1],__temp__)
                        self.parameters.append((self.__scaling_options.get(key)[0],__temp__))
                    except configparser.NoOptionError:
                        pass
        except AttributeError:
            pass

        # Handle extra parameters supplied on command-line
        try:
            if self.args.extra_parameters is not None:
                self.__temp_dict = dict(self.parameters)
                self.__temp_dict.update(dict(self.args.extra_parameters))
                self.__dictlist = []
                for key, value in self.__temp_dict.items():
                    temp = [str(key),str(value)]
                    self.__dictlist.append(temp)
                self.parameters = self.__dictlist
        except AttributeError:
            pass



from __future__ import print_function
from __future__ import absolute_import
# Copyright 2013-2018 Amazon.com, Inc. or its affiliates. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the 'License'). You may not use this file except in compliance with the
# License. A copy of the License is located at
#
# http://aws.amazon.com/apache2.0/
#
# or in the 'LICENSE.txt' file accompanying this file. This file is distributed on an 'AS IS' BASIS, WITHOUT WARRANTIES
# OR CONDITIONS OF ANY KIND, express or implied. See the License for the specific language governing permissions and
# limitations under the License.

from future import standard_library
standard_library.install_aliases()
from builtins import input
import configparser
import sys
import boto3
import os
import logging
import stat
import errno

from . import cfnconfig

logger = logging.getLogger('cfncluster.cfncluster')
unsupported_regions = ['ap-northeast-3']

def prompt(prompt, default_value=None, hidden=False, options=None):
    if hidden and default_value is not None:
        user_prompt = prompt + ' [*******' + default_value[-4:] + ']: '
    else:
        user_prompt = prompt + ' ['
        if default_value is not None:
            user_prompt = user_prompt + default_value + ']: '
        else:
            user_prompt = user_prompt + ']: '

    if isinstance(options, list):
        print('Acceptable Values for %s: ' % prompt)
        for o in options:
            print('    %s' % o)

    var = input(user_prompt)

    if var == '':
        return default_value
    else:
        return var.strip()

def get_regions():
    ec2 = boto3.client('ec2')
    regions = ec2.describe_regions().get('Regions')
    return [region.get('RegionName') for region in regions if region.get('RegionName') not in unsupported_regions]

def ec2_conn(aws_access_key_id, aws_secret_access_key, aws_region_name):
    if aws_region_name:
        region = aws_region_name
    elif os.environ.get('AWS_DEFAULT_REGION'):
        region = os.environ.get('AWS_DEFAULT_REGION')
    else:
        region = 'us-east-1'

    ec2 = boto3.client('ec2', region_name=region,
                       aws_access_key_id=aws_access_key_id,
                       aws_secret_access_key=aws_secret_access_key)
    return ec2

def list_keys(aws_access_key_id, aws_secret_access_key, aws_region_name):
    conn = ec2_conn(aws_access_key_id, aws_secret_access_key, aws_region_name)
    keypairs = conn.describe_key_pairs()
    keynames = []
    for key in keypairs.get('KeyPairs'):
        keynames.append(key.get('KeyName'))

    if not keynames:
        print('ERROR: No Key Pairs found in region ' + aws_region_name)
        print('Please create an EC2 Key Pair before continuing')
        sys.exit(1)

    return keynames

def list_vpcs(aws_access_key_id, aws_secret_access_key, aws_region_name):
    conn = ec2_conn(aws_access_key_id, aws_secret_access_key, aws_region_name)
    vpcs = conn.describe_vpcs()
    vpcids = []
    for vpc in vpcs.get('Vpcs'):
        vpcids.append(vpc.get('VpcId'))

    if not vpcids:
        print('ERROR: No VPCs found in region ' + aws_region_name)
        print('Please create a VPC before continuing')
        sys.exit(1)

    return vpcids

def list_subnets(aws_access_key_id, aws_secret_access_key, aws_region_name, vpc_id):
    conn = ec2_conn(aws_access_key_id, aws_secret_access_key, aws_region_name)
    subnets = conn.describe_subnets(Filters=[{'Name': 'vpcId', 'Values': [vpc_id]}])
    subnetids = []
    for subnet in subnets.get('Subnets'):
        subnetids.append(subnet.get('SubnetId'))

    if not subnetids:
        print('ERROR: No Subnets found in region ' + aws_region_name)
        print('Please create a VPC Subnet before continuing')
        sys.exit(1)

    return subnetids

def configure(args):

    # Determine config file name based on args or default
    if args.config_file is not None:
        config_file = args.config_file
    else:
        config_file = os.path.expanduser(os.path.join('~', '.cfncluster', 'config'))

    config = configparser.ConfigParser()

    # Check if configuration file exists
    if os.path.isfile(config_file):
        config.read(config_file)
        config_read = True

    # Prompt for required values, using existing as defaults
    cluster_template = prompt('Cluster Template', config.get('global', 'cluster_template') if config.has_option('global', 'cluster_template') else 'default')
    aws_access_key_id = prompt('AWS Access Key ID', config.get('aws', 'aws_access_key_id') if config.has_option('aws', 'aws_access_key_id') else None, True)
    aws_secret_access_key = prompt('AWS Secret Access Key ID', config.get('aws', 'aws_secret_access_key') if config.has_option('aws', 'aws_secret_access_key') else None, True)

    # Use built in boto regions as an available option
    aws_region_name = prompt('AWS Region ID', config.get('aws', 'aws_region_name') if config.has_option('aws', 'aws_region_name') else None, options=get_regions())
    vpcname = prompt('VPC Name', config.get('cluster ' + cluster_template, 'vpc_settings') if config.has_option('cluster ' + cluster_template, 'vpc_settings') else 'public')

    # Query EC2 for available keys as options
    key_name = prompt('Key Name', config.get('cluster ' + cluster_template, 'key_name') if config.has_option('cluster ' + cluster_template, 'key_name') else None, options=list_keys(aws_access_key_id, aws_secret_access_key, aws_region_name))
    vpc_id = prompt('VPC ID', config.get('vpc ' + vpcname, 'vpc_id') if config.has_option('vpc ' + vpcname, 'vpc_id') else None, options=list_vpcs(aws_access_key_id, aws_secret_access_key, aws_region_name))
    master_subnet_id = prompt('Master Subnet ID', config.get('vpc ' + vpcname, 'master_subnet_id') if config.has_option('vpc ' + vpcname, 'master_subnet_id') else None, options=list_subnets(aws_access_key_id, aws_secret_access_key, aws_region_name, vpc_id))

    # Dictionary of values we want to set
    s_global = { '__name__': 'global', 'cluster_template': cluster_template, 'update_check': 'true', 'sanity_check': 'true' }
    s_aws = { '__name__': 'aws', 'aws_access_key_id': aws_access_key_id, 'aws_secret_access_key': aws_secret_access_key, 'aws_region_name': aws_region_name }
    s_cluster = { '__name__': 'cluster ' + cluster_template, 'key_name': key_name, 'vpc_settings': vpcname }
    s_vpc = { '__name__': 'vpc ' + vpcname, 'vpc_id': vpc_id, 'master_subnet_id': master_subnet_id }

    sections = [s_aws, s_cluster, s_vpc, s_global]

    # Loop through the configuration sections we care about
    for section in sections:
        try:
            config.add_section(section['__name__'])
        except configparser.DuplicateSectionError:
            pass
        for key, value in section.items():
            # Only update configuration if not set
            if value is not None and key is not '__name__':
                config.set(section['__name__'], key, value)

    # ensure that the directory for the config file exists (because
    # ~/.cfncluster is likely not to exist on first usage)
    try:
        os.makedirs(os.path.dirname(config_file))
    except OSError as e:
        if e.errno != errno.EEXIST:
            raise # can safely ignore EEXISTS for this purpose...

    # Write configuration to disk
    open(config_file,'a').close()
    os.chmod(config_file, stat.S_IRUSR | stat.S_IWUSR)
    with open(config_file,'w') as cf:
        config.write(cf)

    # Verify the configuration
    cfnconfig.CfnClusterConfig(args)




from __future__ import print_function
# Copyright 2013-2014 Amazon.com, Inc. or its affiliates. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License"). You may not use this file except in compliance with the
# License. A copy of the License is located at
#
# http://aws.amazon.com/apache2.0/
#
# or in the "LICENSE.txt" file accompanying this file. This file is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES
# OR CONDITIONS OF ANY KIND, express or implied. See the License for the specific language governing permissions and
# limitations under the License.

from future import standard_library
standard_library.install_aliases()
__author__ = 'dougalb'

import boto3
import urllib.request, urllib.error, urllib.parse
from urllib.parse import urlparse
import sys
from botocore.exceptions import ClientError

def check_resource(region, aws_access_key_id, aws_secret_access_key, resource_type,resource_value):

    # Loop over all supported resource checks
    # EC2 KeyPair
    if resource_type == 'EC2KeyPair':
        try:
            ec2 = boto3.client('ec2', region_name=region,
                                        aws_access_key_id=aws_access_key_id,
                                        aws_secret_access_key=aws_secret_access_key)
            test = ec2.describe_key_pairs(KeyNames=[resource_value])
        except ClientError as e:
            print('Config sanity error: %s' % e.response.get('Error').get('Message'))
            sys.exit(1)
    if resource_type == 'EC2IAMRoleName':
        try:
            iam = boto3.client('iam', region_name=region,
                                aws_access_key_id=aws_access_key_id,
                                aws_secret_access_key=aws_secret_access_key)

            arn = iam.get_role(RoleName=resource_value).get('Role').get('Arn')
            accountid = boto3.client('sts').get_caller_identity().get('Account')

            iam_policy = [(['ec2:DescribeVolumes', 'ec2:AttachVolume', 'ec2:DescribeInstanceAttribute', 'ec2:DescribeInstanceStatus', 'ec2:DescribeInstances'], "*"),
                        (['dynamodb:ListTables'], "*"),
                        (['sqs:SendMessage', 'sqs:ReceiveMessage', 'sqs:ChangeMessageVisibility', 'sqs:DeleteMessage', 'sqs:GetQueueUrl'], "arn:aws:sqs:%s:%s:cfncluster-*" % (region, accountid)),
                        (['autoscaling:DescribeAutoScalingGroups', 'autoscaling:TerminateInstanceInAutoScalingGroup', 'autoscaling:SetDesiredCapacity'], "*"),
                        (['cloudwatch:PutMetricData'], "*"),
                        (['dynamodb:PutItem', 'dynamodb:Query', 'dynamodb:GetItem', 'dynamodb:DeleteItem', 'dynamodb:DescribeTable'], "arn:aws:dynamodb:%s:%s:table/cfncluster-*" % (region, accountid)),
                        (['sqs:ListQueues'], "*"),
                        (['logs:*'], "arn:aws:logs:*:*:*")]

            for actions, resource_arn in iam_policy:
                response = iam.simulate_principal_policy(PolicySourceArn=arn, ActionNames=actions, ResourceArns=[resource_arn])
                for decision in response.get("EvaluationResults"):
                    if decision.get("EvalDecision") != "allowed":
                        print("IAM role error on user provided role %s: action %s is %s" %
                              (resource_value, decision.get("EvalActionName"), decision.get("EvalDecision")))
                        print("See https://cfncluster.readthedocs.io/en/latest/iam.html")
                        sys.exit(1)
        except ClientError as e:
            print('Config sanity error: %s' % e.response.get('Error').get('Message'))
            sys.exit(1)
    # VPC Id
    elif resource_type == 'VPC':
        try:

            ec2 = boto3.client('ec2', region_name=region,
                                        aws_access_key_id=aws_access_key_id,
                                        aws_secret_access_key=aws_secret_access_key)
            test = ec2.describe_vpcs(VpcIds=[resource_value])
        except ClientError as e:
            print('Config sanity error: %s' % e.response.get('Error').get('Message'))
            sys.exit(1)
        # Check for DNS support in the VPC
        if not ec2.describe_vpc_attribute(VpcId=resource_value, Attribute='enableDnsSupport')\
                .get('EnableDnsSupport').get('Value'):
            print("DNS Support is not enabled in %s" % resource_value)
            sys.exit(1)
        if not ec2.describe_vpc_attribute(VpcId=resource_value, Attribute='enableDnsHostnames')\
                .get('EnableDnsHostnames').get('Value'):
            print("DNS Hostnames not enabled in %s" % resource_value)
            sys.exit(1)
    # VPC Subnet Id
    elif resource_type == 'VPCSubnet':
        try:
            ec2 = boto3.client('ec2', region_name=region,
                                        aws_access_key_id=aws_access_key_id,
                                        aws_secret_access_key=aws_secret_access_key)
            test = ec2.describe_subnets(SubnetIds=[resource_value])
        except ClientError as e:
            print('Config sanity error: %s' % e.response.get('Error').get('Message'))
            sys.exit(1)
    # VPC Security Group
    elif resource_type == 'VPCSecurityGroup':
        try:
            ec2 = boto3.client('ec2', region_name=region,
                               aws_access_key_id=aws_access_key_id,
                               aws_secret_access_key=aws_secret_access_key)
            test = ec2.describe_security_groups(GroupIds=[resource_value])
        except ClientError as e:
            print('Config sanity error: %s' % e.response.get('Error').get('Message'))
            sys.exit(1)
    # EC2 AMI Id
    elif resource_type == 'EC2Ami':
        try:
            ec2 = boto3.client('ec2', region_name=region,
                               aws_access_key_id=aws_access_key_id,
                               aws_secret_access_key=aws_secret_access_key)
            test = ec2.describe_images(ImageIds=[resource_value])
        except ClientError as e:
            print('Config sanity error: %s' % e.response.get('Error').get('Message'))
            sys.exit(1)
    # EC2 Placement Group
    elif resource_type == 'EC2PlacementGroup':
        if resource_value == 'DYNAMIC':
            pass
        else:
            try:
                ec2 = boto3.client('ec2', region_name=region,
                                   aws_access_key_id=aws_access_key_id,
                                   aws_secret_access_key=aws_secret_access_key)
                test = ec2.describe_placement_groups(GroupNames=[resource_value])
            except ClientError as e:
                print('Config sanity error: %s' % e.response.get('Error').get('Message'))
                sys.exit(1)
    # URL
    elif resource_type == 'URL':
        scheme = urlparse(resource_value).scheme
        if scheme == 's3':
            pass
        else:
            try:
                urllib.request.urlopen(resource_value)
            except urllib.error.HTTPError as e:
                print('Config sanity error:', resource_value, e.code, e.reason)
                sys.exit(1)
            except urllib.error.URLError as e:
                print('Config sanity error:', resource_value, e.reason)
                sys.exit(1)
    # EC2 EBS Snapshot Id
    elif resource_type == 'EC2Snapshot':
        try:
            ec2 = boto3.client('ec2', region_name=region,
                               aws_access_key_id=aws_access_key_id,
                               aws_secret_access_key=aws_secret_access_key)
            test = ec2.describe_snapshots(SnapshotIds=[resource_value])
        except ClientError as e:
            print('Config sanity error: %s' % e.response.get('Error').get('Message'))
            sys.exit(1)
    # EC2 EBS Volume Id
    elif resource_type == 'EC2Volume':
        try:
            ec2 = boto3.client('ec2', region_name=region,
                               aws_access_key_id=aws_access_key_id,
                               aws_secret_access_key=aws_secret_access_key)
            test = ec2.describe_volumes(VolumeIds=[resource_value]).get('Volumes')[0]
            if test.get('State') != 'available':
                print('Volume %s is in state \'%s\' not \'available\'' % (resource_value, test.get('State')))
                sys.exit(1)
        except ClientError as e:
            if e.response.get('Error').get('Message').endswith('parameter volumes is invalid. Expected: \'vol-...\'.'):
                print('Config sanity error: volume %s does not exist.' % resource_value)
                sys.exit(1)
            print('Config sanity error: %s' % e.response.get('Error').get('Message'))
            sys.exit(1)



# Copyright 2013-2016 Amazon.com, Inc. or its affiliates. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License"). You may not use this file except in compliance with the
# License. A copy of the License is located at
#
# http://aws.amazon.com/apache2.0/
#
# or in the "LICENSE.txt" file accompanying this file. This file is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES
# OR CONDITIONS OF ANY KIND, express or implied. See the License for the specific language governing permissions and
# limitations under the License.

import os, sys
from setuptools import setup, find_packages

# Utility function to read the README file.
# Used for the long_description.  It's nice, because now 1) we have a top level
# README file and 2) it's easier to type in the README file than to put a raw
# string in below ...
def read(fname):
    return open(os.path.join(os.path.dirname(__file__), fname)).read()

console_scripts = ['cfncluster = cfncluster.cli:main']
version = "1.5.1"
requires = ['boto3>=1.7.33', 'awscli>=1.11.175', 'future>=0.16.0']

if sys.version_info[:2] == (2, 6):
    # For python2.6 we have to require argparse since it
    # was not in stdlib until 2.7.
    requires.append('argparse>=1.4.0')

if sys.version_info[0] == 2:
    requires.append('configparser>=3.5.0')

setup(
    name = "cfncluster",
    version = version,
    author = "Dougal Ballantyne",
    author_email = "dougalb@amazon.com",
    description = ("A simple tool to launch and manage HPC clusters as CloudFormation stacks."),
    url = ("https://github.com/awslabs/cfncluster"),
    license = "Apache License 2.0",
    packages = find_packages(),
    install_requires = requires,
    entry_points=dict(console_scripts=console_scripts),
    include_package_data = True,
    zip_safe = False,
    package_data = {
        '' : ['examples/config'],
    },
    long_description=read('README'),
    classifiers=[
        "Development Status :: 5 - Production/Stable",
        "Environment :: Console",
        "Programming Language :: Python",
        "Topic :: Scientific/Engineering",
        "License :: OSI Approved :: Apache Software License",
    ],
)



from __future__ import absolute_import
# Copyright 2013-2018 Amazon.com, Inc. or its affiliates. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License"). You may not use this file except in compliance with the
# License. A copy of the License is located at
#
# http://aws.amazon.com/apache2.0/
#
# or in the "LICENSE.txt" file accompanying this file. This file is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES
# OR CONDITIONS OF ANY KIND, express or implied. See the License for the specific language governing permissions and
# limitations under the License.

import os
import argparse
import logging
import platform
import json
import sys
import errno

from . import cfncluster
from . import easyconfig


def create(args):
    cfncluster.create(args)

def configure(args):
    easyconfig.configure(args)

def status(args):
    cfncluster.status(args)

def list(args):
    cfncluster.list(args)

def delete(args):
    cfncluster.delete(args)

def instances(args):
    cfncluster.instances(args)

def update(args):
    cfncluster.update(args)

def version(args):
    cfncluster.version(args)

def start(args):
    cfncluster.start(args)

def stop(args):
    cfncluster.stop(args)

def config_logger():
    logger = logging.getLogger('cfncluster.cfncluster')
    logger.setLevel(logging.DEBUG)

    ch = logging.StreamHandler(sys.stdout)
    ch.setLevel(logging.INFO)
    ch.setFormatter(logging.Formatter('%(message)s'))
    logger.addHandler(ch)

    logfile = os.path.expanduser(os.path.join('~', '.cfncluster', 'cfncluster-cli.log'))
    try:
        os.makedirs(os.path.dirname(logfile))
    except OSError as e:
        if e.errno != errno.EEXIST:
            raise # can safely ignore EEXISTS for this purpose...

    fh = logging.FileHandler(logfile)
    fh.setLevel(logging.DEBUG)
    fh.setFormatter(logging.Formatter('%(asctime)s - %(levelname)s - %(name)s - %(message)s'))
    logger.addHandler(fh)

def main():
    config_logger()

    logger = logging.getLogger('cfncluster.cfncluster')
    logger.debug("CfnCluster cli starting")

    parser = argparse.ArgumentParser(description='cfncluster is a tool to launch and manage a cluster.')
    parser.add_argument("--config", "-c", dest="config_file", help='specify a alternative config file')
    parser.add_argument( "--region", "-r", dest="region", help='specify a specific region to connect to',
                        default=None)
    parser.add_argument( "--nowait", "-nw", dest="nowait", action='store_true',
                        help='do not wait for stack events, after executing stack command')

    subparsers = parser.add_subparsers()
    subparsers.required = True
    subparsers.dest = 'command'

    pcreate = subparsers.add_parser('create', help='creates a cluster')
    pcreate.add_argument("cluster_name", type=str, default=None,
                        help='create a cfncluster with the provided name.')
    pcreate.add_argument("--norollback", "-nr", action='store_true', dest="norollback", default=False,
                         help='disable stack rollback on error')
    pcreate.add_argument("--template-url", "-u", type=str, dest="template_url", default=None,
                         help='specify a URL for a custom cloudformation template')
    pcreate.add_argument("--cluster-template", "-t", type=str, dest="cluster_template", default=None,
                         help='specify a specific cluster template to use')
    pcreate.add_argument("--extra-parameters", "-p", type=json.loads, dest="extra_parameters", default=None,
                         help='add extra parameters to stack create')
    pcreate.add_argument("--tags", "-g", type=json.loads, dest="tags", default=None,
                         help='tags to be added to the stack')
    pcreate.set_defaults(func=create)

    pupdate = subparsers.add_parser('update', help='update a running cluster')
    pupdate.add_argument("cluster_name", type=str, default=None,
                        help='update a cfncluster with the provided name.')
    pupdate.add_argument("--norollback", "-nr", action='store_true', dest="norollback", default=False,
                         help='disable stack rollback on error')
    pupdate.add_argument("--template-url", "-u", type=str, dest="template_url", default=None,
                         help='specify a URL for a custom cloudformation template')
    pupdate.add_argument("--cluster-template", "-t", type=str, dest="cluster_template", default=None,
                         help='specify a specific cluster template to use')
    pupdate.add_argument("--extra-parameters", "-p", type=str, dest="extra_parameters", default=None,
                         help='add extra parameters to stack update')
    pupdate.add_argument("--reset-desired", "-rd", action='store_true', dest="reset_desired", default=False,
                         help='reset the current ASG desired capacity to initial config values')
    pupdate.set_defaults(func=update)

    pdelete = subparsers.add_parser('delete', help='delete a cluster')
    pdelete.add_argument("cluster_name", type=str, default=None,
                        help='delete a cfncluster with the provided name.')
    pdelete.set_defaults(func=delete)

    pstart = subparsers.add_parser('start', help='start the compute-fleet that has been stopped')
    pstart.add_argument("cluster_name", type=str, default=None,
                        help='starts the compute-fleet of the provided cluster name.')
    pstart.set_defaults(func=start)

    pstop = subparsers.add_parser('stop', help='stop the compute-fleet, but leave the MasterServer running for debugging/development')
    pstop.add_argument("cluster_name", type=str, default=None,
                        help='stops the compute-fleet of the provided cluster name.')
    pstop.set_defaults(func=stop)

    pstatus = subparsers.add_parser('status', help='pull the current status of the cluster')
    pstatus.add_argument("cluster_name", type=str, default=None,
                        help='show the status of cfncluster with the provided name.')
    pstatus.set_defaults(func=status)

    plist = subparsers.add_parser('list', help='display a list of stacks associated with cfncluster')
    plist.set_defaults(func=list)

    pinstances = subparsers.add_parser('instances', help='display a list of all instances in a cluster')
    pinstances.add_argument("cluster_name", type=str, default=None,
                        help='show the status of cfncluster with the provided name.')
    pinstances.set_defaults(func=instances)

    pconfigure = subparsers.add_parser('configure', help='creating initial cfncluster configuration')
    pconfigure.set_defaults(func=configure)

    pversion = subparsers.add_parser('version', help='display version of cfncluster')
    pversion.set_defaults(func=version)

    args = parser.parse_args()
    logger.debug(args)
    args.func(args)



from __future__ import print_function
from __future__ import absolute_import
# Copyright 2013-2018 Amazon.com, Inc. or its affiliates. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License"). You may not use this file except in compliance with the
# License. A copy of the License is located at
#
# http://aws.amazon.com/apache2.0/
#
# or in the "LICENSE.txt" file accompanying this file. This file is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES
# OR CONDITIONS OF ANY KIND, express or implied. See the License for the specific language governing permissions and
# limitations under the License.

from builtins import str
import sys
import time
import logging
import boto3
from botocore.exceptions import ClientError

from . import cfnconfig

logger = logging.getLogger('cfncluster.cfncluster')

def version(args):
    config = cfnconfig.CfnClusterConfig(args)
    logger.info(config.version)

def create(args):
    logger.info('Beginning cluster creation for cluster: %s' % (args.cluster_name))
    logger.debug('Building cluster config based on args %s' % str(args))

    # Build the config based on args
    config = cfnconfig.CfnClusterConfig(args)

    # Set the ComputeWaitConditionCount parameter to match InitialQueueSize
    try:
        i = [p[0] for p in config.parameters].index('InitialQueueSize')
        initial_queue_size = config.parameters[i][1]
        config.parameters.append(('ComputeWaitConditionCount', initial_queue_size))
    except ValueError:
        pass

    # Get the MasterSubnetId and use it to determine AvailabilityZone
    try:
        i = [p[0] for p in config.parameters].index('MasterSubnetId')
        master_subnet_id = config.parameters[i][1]
        try:
            ec2 = boto3.client('ec2', region_name=config.region,
                                 aws_access_key_id=config.aws_access_key_id,
                                 aws_secret_access_key=config.aws_secret_access_key)
            availability_zone = ec2.describe_subnets(SubnetIds=[master_subnet_id])\
                .get('Subnets')[0]\
                .get('AvailabilityZone')
        except ClientError as e:
            logger.critical(e.response.get('Error').get('Message'))
            sys.stdout.flush()
            sys.exit(1)
        config.parameters.append(('AvailabilityZone', availability_zone))
    except ValueError:
        pass

    capabilities = ["CAPABILITY_IAM"]
    try:
        cfn = boto3.client('cloudformation', region_name=config.region,
                           aws_access_key_id=config.aws_access_key_id,
                           aws_secret_access_key=config.aws_secret_access_key)
        stack_name = 'cfncluster-' + args.cluster_name
        logger.info("Creating stack named: " + stack_name)

        cfn_params = [{'ParameterKey': param[0], 'ParameterValue': param[1]} for param in config.parameters]
        tags = [{'Key': t, 'Value': config.tags[t]} for t in config.tags]

        stack = cfn.create_stack(StackName=stack_name,
                                 TemplateURL=config.template_url,
                                 Parameters=cfn_params,
                                 Capabilities=capabilities,
                                 DisableRollback=args.norollback, Tags=tags)
        logger.debug('StackId: %s' % (stack.get('StackId')))

        status = cfn.describe_stacks(StackName=stack_name).get("Stacks")[0].get('StackStatus')

        if not args.nowait:
            resource_status = ''
            while status == 'CREATE_IN_PROGRESS':
                status = cfn.describe_stacks(StackName=stack_name).get("Stacks")[0].get('StackStatus')
                events = cfn.describe_stack_events(StackName=stack_name).get('StackEvents')[0]
                resource_status = ('Status: %s - %s' % (events.get('LogicalResourceId'), events.get('ResourceStatus'))).ljust(80)
                sys.stdout.write('\r%s' % resource_status)
                sys.stdout.flush()
                time.sleep(5)
            # print the last status update in the logs
            if resource_status != '':
                logger.debug(resource_status)

            if status != 'CREATE_COMPLETE':
                logger.critical('\nCluster creation failed.  Failed events:')
                events = cfn.describe_stack_events(StackName=stack_name).get('StackEvents')
                for event in events:
                    if event.get('ResourceStatus') == 'CREATE_FAILED':
                        logger.info("  - %s %s %s" %
                                    (event.get('ResourceType'), event.get('LogicalResourceId'),
                                     event.get('ResourceStatusReason')))
            logger.info('')
            outputs = cfn.describe_stacks(StackName=stack_name).get("Stacks")[0].get('Outputs', [])
            for output in outputs:
                logger.info("%s: %s" % (output.get('OutputKey'), output.get('OutputValue')))
        else:
            status = cfn.describe_stacks(StackName=stack_name).get("Stacks")[0].get('StackStatus')
            logger.info('Status: %s' % status)
    except ClientError as e:
        logger.critical(e.response.get('Error').get('Message'))
        sys.stdout.flush()
        sys.exit(1)
    except KeyboardInterrupt:
        logger.info('\nExiting...')
        sys.exit(0)
    except Exception as e:
        logger.critical(e)
        sys.exit(1)


def update(args):
    logger.info('Updating: %s' % (args.cluster_name))
    stack_name = ('cfncluster-' + args.cluster_name)
    config = cfnconfig.CfnClusterConfig(args)
    capabilities = ["CAPABILITY_IAM"]

    cfn = boto3.client('cloudformation', region_name=config.region,
                       aws_access_key_id=config.aws_access_key_id,
                       aws_secret_access_key=config.aws_secret_access_key)

    asg = boto3.client('autoscaling', region_name=config.region,
                       aws_access_key_id=config.aws_access_key_id,
                       aws_secret_access_key=config.aws_secret_access_key)

    if not args.reset_desired:
        asg_name = get_asg_name(stack_name, config)
        desired_capacity = asg.describe_auto_scaling_groups(AutoScalingGroupNames=[asg_name])\
            .get('AutoScalingGroups')[0]\
            .get('DesiredCapacity')
        config.parameters.append(('InitialQueueSize', desired_capacity))

    # Get the MasterSubnetId and use it to determine AvailabilityZone
    try:
        i = [p[0] for p in config.parameters].index('MasterSubnetId')
        master_subnet_id = config.parameters[i][1]
        try:
            ec2 = boto3.client('ec2', region_name=config.region,
                               aws_access_key_id=config.aws_access_key_id,
                               aws_secret_access_key=config.aws_secret_access_key)
            availability_zone = ec2.describe_subnets(SubnetIds=[master_subnet_id]) \
                .get('Subnets')[0] \
                .get('AvailabilityZone')
        except ClientError as e:
            logger.critical(e.response.get('Error').get('Message'))
            sys.exit(1)
        config.parameters.append(('AvailabilityZone', availability_zone))
    except ValueError:
        pass

    try:
        logger.debug((config.template_url, config.parameters))

        cfn_params = [{'ParameterKey': param[0], 'ParameterValue': str(param[1])} for param in config.parameters]
        cfn.update_stack(StackName=stack_name,TemplateURL=config.template_url,
                         Parameters=cfn_params, Capabilities=capabilities)
        status = cfn.describe_stacks(StackName=stack_name).get("Stacks")[0].get('StackStatus')
        if not args.nowait:
            while status == 'UPDATE_IN_PROGRESS':
                status = cfn.describe_stacks(StackName=stack_name).get("Stacks")[0].get('StackStatus')
                events = cfn.describe_stack_events(StackName=stack_name).get('StackEvents')[0]
                resource_status = ('Status: %s - %s' % (events.get('LogicalResourceId'), events.get('ResourceStatus'))).ljust(80)
                sys.stdout.write('\r%s' % resource_status)
                sys.stdout.flush()
                time.sleep(5)
        else:
            status = cfn.describe_stacks(StackName=stack_name).get("Stacks")[0].get('StackStatus')
            logger.info('Status: %s' % status)
    except ClientError as e:
        logger.critical(e.response.get('Error').get('Message'))
        sys.exit(1)
    except KeyboardInterrupt:
        logger.info('\nExiting...')
        sys.exit(0)

def start(args):
    # Set resource limits on compute fleet to min/max/desired = 0/max/0
    logger.info('Starting compute fleet : %s' % args.cluster_name)
    stack_name = ('cfncluster-' + args.cluster_name)
    config = cfnconfig.CfnClusterConfig(args)

    # Set asg limits
    max_queue_size = [param[1] for param in config.parameters if param[0] == 'MaxQueueSize']
    max_queue_size = int(max_queue_size[0] if len(max_queue_size) > 0 else 10)
    desired_queue_size = [param[1] for param in config.parameters if param[0] == 'InitialQueueSize']
    desired_queue_size = int(desired_queue_size[0] if len(desired_queue_size) > 0 else 2)
    min_queue_size = [desired_queue_size for param in config.parameters if param[0] == 'MaintainInitialSize' and param[1] == "true"]
    min_queue_size = int(min_queue_size[0] if len(min_queue_size) > 0 else 0)

    asg_name = get_asg_name(stack_name=stack_name, config=config)
    set_asg_limits(asg_name=asg_name, config=config, min=min_queue_size, max=max_queue_size, desired=desired_queue_size)

def stop(args):
    # Set resource limits on compute fleet to min/max/desired = 0/0/0
    logger.info('Stopping compute fleet : %s' % args.cluster_name)
    stack_name = ('cfncluster-' + args.cluster_name)
    config = cfnconfig.CfnClusterConfig(args)

    # Set Resource limits
    asg_name = get_asg_name(stack_name=stack_name, config=config)
    set_asg_limits(asg_name=asg_name, config=config, min=0, max=0, desired=0)

def list(args):
    config = cfnconfig.CfnClusterConfig(args)
    cfn = boto3.client('cloudformation', region_name=config.region,
                       aws_access_key_id=config.aws_access_key_id,
                       aws_secret_access_key=config.aws_secret_access_key)
    try:
        stacks = cfn.describe_stacks().get('Stacks')
        for stack in stacks:
            if stack.get('StackName').startswith('cfncluster-'):
                logger.info('%s' % (stack.get('StackName')[11:]))
    except ClientError as e:
        logger.critical(e.response.get('Error').get('Message'))
        sys.exit(1)
    except KeyboardInterrupt:
        logger.info('Exiting...')
        sys.exit(0)

def get_master_server_id(stack_name, config):
    # returns the physical id of the master server
    # if no master server returns []
    cfn = boto3.client('cloudformation', region_name=config.region,
                       aws_access_key_id=config.aws_access_key_id,
                       aws_secret_access_key=config.aws_secret_access_key)

    try:
        resources = cfn.describe_stack_resource(StackName=stack_name, LogicalResourceId='MasterServer')
        return resources.get('StackResourceDetail').get('PhysicalResourceId')
    except ClientError as e:
        logger.critical(e.response.get('Error').get('Message'))
        sys.exit(1)


def poll_master_server_state(stack_name, config):
    ec2 = boto3.client('ec2', region_name=config.region,
                       aws_access_key_id=config.aws_access_key_id,
                       aws_secret_access_key=config.aws_secret_access_key)

    master_id = get_master_server_id(stack_name, config)

    try:
        instance = ec2.describe_instance_status(InstanceIds=[master_id]).get('InstanceStatuses')[0]
        state = instance.get('InstanceState').get('Name')
        sys.stdout.write('\rMasterServer: %s' % state.upper())
        sys.stdout.flush()
        while state not in ['running', 'stopped', 'terminated', 'shutting-down']:
            time.sleep(5)
            state = ec2.describe_instance_status(InstanceIds=[master_id]).get('InstanceStatuses')[0].get('InstanceState').get('Name')
            status = ('\r\033[KMasterServer: %s' % state.upper())
            sys.stdout.write(status)
            sys.stdout.flush()
        if state in ['terminated', 'shutting-down']:
            logger.info("State: %s is irrecoverable. Cluster needs to be re-created.")
            sys.exit(1)
        status = ('\rMasterServer: %s\n' % state.upper())
        sys.stdout.write(status)
        sys.stdout.flush()
    except ClientError as e:
        logger.critical(e.response.get('Error').get('Message'))
        sys.stdout.flush()
        sys.exit(1)
    except KeyboardInterrupt:
        logger.info('\nExiting...')
        sys.exit(0)

    return state

def get_ec2_instances(stack, config):
    cfn = boto3.client('cloudformation', region_name=config.region,
                       aws_access_key_id=config.aws_access_key_id,
                       aws_secret_access_key=config.aws_secret_access_key)

    try:
        resources = cfn.describe_stack_resources(StackName=stack).get('StackResources')
    except ClientError as e:
        logger.critical(e.response.get('Error').get('Message'))
        sys.stdout.flush()
        sys.exit(1)

    temp_instances = [r for r in resources if r.get('ResourceType') == 'AWS::EC2::Instance']

    instances = []
    for instance in temp_instances:
        instances.append([instance.get('LogicalResourceId'),instance.get('PhysicalResourceId')])

    return instances

def get_asg_name(stack_name, config):
    cfn = boto3.client('cloudformation', region_name=config.region,
                       aws_access_key_id=config.aws_access_key_id,
                       aws_secret_access_key=config.aws_secret_access_key)
    try:
        resources = cfn.describe_stack_resources(StackName=stack_name).get('StackResources')
        return [r for r in resources if r.get('LogicalResourceId') == 'ComputeFleet'][0].get('PhysicalResourceId')
    except ClientError as e:
        logger.critical(e.response.get('Error').get('Message'))
        sys.stdout.flush()
        sys.exit(1)
    except IndexError:
        logger.critical("Stack %s does not have a ComputeFleet" % stack_name)
        sys.exit(1)

def set_asg_limits(asg_name, config, min, max, desired):
    asg = boto3.client('autoscaling', region_name=config.region,
                 aws_access_key_id=config.aws_access_key_id,
                 aws_secret_access_key=config.aws_secret_access_key)

    asg.update_auto_scaling_group(AutoScalingGroupName=asg_name, MinSize=min, MaxSize=max,
                                  DesiredCapacity=desired)

def get_asg_instances(stack, config):
    asg = boto3.client('autoscaling', region_name=config.region,
                       aws_access_key_id=config.aws_access_key_id,
                       aws_secret_access_key=config.aws_secret_access_key)

    asg_name = get_asg_name(stack, config)
    asg = asg.describe_auto_scaling_groups(AutoScalingGroupNames=[asg_name]).get('AutoScalingGroups')[0]
    name = [tag.get('Value') for tag in asg.get('Tags') if tag.get('Key') == 'aws:cloudformation:logical-id'][0]

    temp_instances = []
    for instance in asg.get('Instances'):
        temp_instances.append([name,instance.get('InstanceId')])

    return temp_instances

def instances(args):
    stack = ('cfncluster-' + args.cluster_name)

    config = cfnconfig.CfnClusterConfig(args)
    instances = []
    instances.extend(get_ec2_instances(stack, config))
    instances.extend(get_asg_instances(stack, config))

    for instance in instances:
        print('%s         %s' % (instance[0],instance[1]))

def status(args):
    stack = ('cfncluster-' + args.cluster_name)
    config = cfnconfig.CfnClusterConfig(args)

    cfn = boto3.client('cloudformation', region_name=config.region,
                       aws_access_key_id=config.aws_access_key_id,
                       aws_secret_access_key=config.aws_secret_access_key)

    try:
        status = cfn.describe_stacks(StackName=stack).get("Stacks")[0].get('StackStatus')
        sys.stdout.write('\rStatus: %s' % status)
        sys.stdout.flush()
        if not args.nowait:
            while status not in ['CREATE_COMPLETE', 'UPDATE_COMPLETE', 'UPDATE_ROLLBACK_COMPLETE',
                                 'ROLLBACK_COMPLETE', 'CREATE_FAILED', 'DELETE_FAILED']:
                time.sleep(5)
                status = cfn.describe_stacks(StackName=stack).get("Stacks")[0].get('StackStatus')
                events = cfn.describe_stack_events(StackName=stack).get('StackEvents')[0]
                resource_status = ('Status: %s - %s' % (events.get('LogicalResourceId'), events.get('ResourceStatus'))).ljust(80)
                sys.stdout.write('\r%s' % resource_status)
                sys.stdout.flush()
            sys.stdout.write('\rStatus: %s\n' % status)
            sys.stdout.flush()
            if status in ['CREATE_COMPLETE', 'UPDATE_COMPLETE']:
                state = poll_master_server_state(stack, config)
                if state == 'running':
                    outputs = cfn.describe_stacks(StackName=stack).get("Stacks")[0].get('Outputs', [])
                    for output in outputs:
                        logger.info("%s: %s" % (output.get('OutputKey'), output.get('OutputValue')))
            elif status in ['ROLLBACK_COMPLETE', 'CREATE_FAILED', 'DELETE_FAILED', 'UPDATE_ROLLBACK_COMPLETE']:
                events = cfn.describe_stack_events(StackName=stack).get('StackEvents')
                for event in events:
                    if event.get('ResourceStatus') in ['CREATE_FAILED', 'DELETE_FAILED', 'UPDATE_FAILED']:
                        logger.info("%s %s %s %s %s" %
                                    (event.get('Timestamp'), event.get('ResourceStatus'),
                                     event.get('ResourceType'), event.get('LogicalResourceId'),
                                     event.get('ResourceStatusReason')))
        else:
            sys.stdout.write('\n')
            sys.stdout.flush()
    except ClientError as e:
        logger.critical(e.response.get('Error').get('Message'))
        sys.stdout.flush()
        sys.exit(1)
    except KeyboardInterrupt:
        logger.info('\nExiting...')
        sys.exit(0)

def delete(args):
    saw_update = False
    logger.info('Deleting: %s' % args.cluster_name)
    stack = ('cfncluster-' + args.cluster_name)

    config = cfnconfig.CfnClusterConfig(args)

    cfn = boto3.client('cloudformation', region_name=config.region,
                       aws_access_key_id=config.aws_access_key_id,
                       aws_secret_access_key=config.aws_secret_access_key)

    try:
        cfn.delete_stack(StackName=stack)
        status = cfn.describe_stacks(StackName=stack).get("Stacks")[0].get('StackStatus')
        saw_update = True
        sys.stdout.write('\rStatus: %s' % status)
        sys.stdout.flush()
        logger.debug('Status: %s' % status)
        if not args.nowait:
            while status == 'DELETE_IN_PROGRESS':
                time.sleep(5)
                status = cfn.describe_stacks(StackName=stack).get("Stacks")[0].get('StackStatus')
                events = cfn.describe_stack_events(StackName=stack).get('StackEvents')[0]
                resource_status = ('Status: %s - %s' % (events.get('LogicalResourceId'), events.get('ResourceStatus'))).ljust(80)
                sys.stdout.write('\r%s' % resource_status)
                sys.stdout.flush()
            sys.stdout.write('\rStatus: %s\n' % status)
            sys.stdout.flush()
            logger.debug('Status: %s' % status)
        else:
            sys.stdout.write('\n')
            sys.stdout.flush()
        if status == 'DELETE_FAILED':
            logger.info('Cluster did not delete successfully. Run \'cfncluster delete %s\' again' % stack)
    except ClientError as e:
        if e.response.get('Error').get('Message').endswith("does not exist"):
            if saw_update:
                logger.info('\nCluster deleted successfully.')
                sys.exit(0)
        logger.critical(e.response.get('Error').get('Message'))
        sys.stdout.flush()
        sys.exit(1)
    except KeyboardInterrupt:
        logger.info('\nExiting...')
        sys.exit(0)

